unit Graphics;

interface

	uses
		QDOffscreen, Globals, Game;

{----------------------------}

{ Pellet array, top 2 rows (65 each row) }
{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
{0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0}

	procedure InitGraphics;
	procedure DrawScreen (entireFlag: boolean);

{----------------------------}

implementation

{----------------------------}

	procedure BuildOffWorld (sizeOfDoc: Rect);

		var
			oldPerm: boolean;
			currDevice: GDHandle;
			currPort: GWorldPtr;
			erry: OSErr;

	begin
		GetGWorld(currPort, currDevice);
		erry := NewGWorld(gameWindow.offGWorld, 16, sizeOfDoc, nil, nil, GWorldFlags(0));

		SetGWorld(gameWindow.offGWorld, nil);
		if (LockPixels(gameWindow.offGWorld^.portPixMap)) then
			begin
				EraseRect(gameWindow.offGWorld^.portRect);
				UnlockPixels(gameWindow.offGWorld^.portPixMap);
			end;

		SetGWorld(currPort, currDevice);
	end;

{----------------------------}

	procedure InitPict_Backdrop;

		var
			resourceRect: Rect;

	begin
		with backgroundPict do
			begin
				if gColorQdAvailable then
					if gameMenus.windowSelection = Small then
						pictH := GetPicture(backdropSmallBwPictID)
					else if gameMenus.windowSelection = Medium then
						pictH := GetPicture(backdropMediumColorPictID)
					else if gameMenus.windowSelection = Large then
						pictH := GetPicture(backdropLargeColorPictID)
					else
						exit(InitPict_Backdrop)
				else
					pictH := GetPicture(backdropSmallBwPictID);

				resourceRect := backgroundPict.pictH^^.picFrame;
				pictWidth := resourceRect.right - resourceRect.left;
				pictHeight := resourceRect.bottom - resourceRect.top;
				pictLeft := round((gameWindow.width - pictWidth) / 2);
				pictTop := round((gameWindow.height - pictHeight) / 2);
				SetRect(pictRect, pictLeft, pictTop, pictLeft + pictWidth, pictTop + pictHeight);
			end;
	end;

{----------------------------}

	procedure InitPict_Pac;

		var
			resourceRect: Rect;
			index: integer;

	begin
		case gameMenus.windowSelection of
			Small: 
				begin
					for index := 1 to 9 do
						with pacmanInfo.pictArray[index] do
							pictH := GetPicture(smallPacman_solid - 1 + index);
					with pacmanInfo do
						begin
							resourceRect := pictArray[2].pictH^^.picFrame;
							pictSize := resourceRect.right - resourceRect.left;
						end;
				end;

			Medium: 
				begin
					for index := 1 to 9 do
						with pacmanInfo.pictArray[index] do
							pictH := GetPicture(mediumPacman_solid - 1 + index);
					with pacmanInfo do
						begin
							resourceRect := pictArray[2].pictH^^.picFrame;
							pictSize := resourceRect.right - resourceRect.left;
						end;
				end;

			Large: 
				begin
					for index := 1 to 9 do
						with pacmanInfo.pictArray[index] do
							pictH := GetPicture(largePacman_solid - 1 + index);
					with pacmanInfo do
						begin
							resourceRect := pictArray[2].pictH^^.picFrame;
							pictSize := resourceRect.right - resourceRect.left;
						end;
				end;

		end;
	end;

{----------------------------}

	procedure InitPict_Pellet;

		var
			resourceRect: Rect;

	begin
		case gameMenus.windowSelection of
			Small: 
				with pelletInfo do
					begin
						pictImage.pictH := GetPicture(mediumPellet);  { couldn't get small pellet to draw }
						resourceRect := pictImage.pictH^^.picFrame;
						pictSize := resourceRect.right - resourceRect.left;
					end;

			Medium: 
				with pelletInfo do
					begin
						pictImage.pictH := GetPicture(mediumPellet);
						resourceRect := pictImage.pictH^^.picFrame;
						pictSize := resourceRect.right - resourceRect.left;
					end;

			Large: 
				with pelletInfo do
					begin
						pictImage.pictH := GetPicture(largePellet);
						resourceRect := pictImage.pictH^^.picFrame;
						pictSize := resourceRect.right - resourceRect.left;
					end;

		end;
	end;

{----------------------------}

	procedure InitGraphics;

	begin
		BuildOffWorld(gameWindow.windowPtr^.portRect);
		InitPict_Backdrop;
		InitPict_Pac;
		InitPict_Pellet;
	end;

{----------------------------}

	procedure DrawStats;

		var
			ticksAsString, pelletsAsString: Str255;
			textRect: Rect;

	begin
		SetRect(textRect, 10, 30, 120, 40);
		FillRect(textRect, black);
		PenPat(white);
		MoveTo(10, 40);
		TextMode(srcXor);
		NumToString(appGlobals.avgDrawCycles, ticksAsString);
		DrawString(concat('ticks/draw:', ticksAsString));

		SetRect(textRect, 10, 45, 120, 57);
		FillRect(textRect, black);
		PenPat(white);
		MoveTo(10, 55);
		TextMode(srcXor);
		NumToString(gameGlobals.pelletsConsumed, pelletsAsString);
		DrawString(concat('pellets:', pelletsAsString));
	end;

{----------------------------}

	procedure DrawPac (offscreenPixMap: PixMap);

		var
			offscreenRect: Rect;
			offscreenBitMapPtr: BitMapPtr;

	begin
{ create an offscreen spot to temorarily copy/render the previous pixels, of the new location }
		offscreenBitMapPtr := BitMapPtr(@offscreenPixMap);

		with pacmanInfo do
			begin

{ restore the previous pixels (erase existing pacman) }
				SetRect(offscreenRect, 10, 100, 10 + pacmanInfo.pictSize, 100 + pacmanInfo.pictSize);
				CopyBits(offscreenBitMapPtr^, offscreenBitMapPtr^, offscreenRect, pacmanInfo.pictRect, srcCopy, nil);

{ each animation frame needs a different image - currently 3 frames for pacman }
				frameNo := frameNo + 1;
				if frameNo > 2 then
					frameNo := 0;

				CollisionHandler;

{ determine new position for pacman }
				SetNewPosition(PacMan);

{ capture existing pixels of new location }
				SetRect(offscreenRect, 10, 100, 10 + pacmanInfo.pictSize, 100 + pacmanInfo.pictSize);
				CopyBits(offscreenBitMapPtr^, offscreenBitMapPtr^, pacmanInfo.pictRect, offscreenRect, srcCopy, nil);

{ draw a white rect around the working area, just for developer's visual reference }
				PenPat(white);
				InsetRect(offscreenRect, -1, -1);
				FrameRect(offscreenrect);

{ draw pacman image at the new location, based on frameNo }
				case direction of
					Left: 
						if frameNo = 0 then
							DrawPicture(pictArray[1].pictH, pictRect)
						else if frameNo = 1 then
							DrawPicture(pictArray[2].pictH, pictRect)
						else
							DrawPicture(pictArray[3].pictH, pictRect);

					Up: 
						if frameNo = 0 then
							DrawPicture(pictArray[1].pictH, pictRect)
						else if frameNo = 1 then
							DrawPicture(pictArray[8].pictH, pictRect)
						else
							DrawPicture(pictArray[9].pictH, pictRect);

					Right: 
						if frameNo = 0 then
							DrawPicture(pictArray[1].pictH, pictRect)
						else if frameNo = 1 then
							DrawPicture(pictArray[4].pictH, pictRect)
						else
							DrawPicture(pictArray[5].pictH, pictRect);

					Down: 
						if frameNo = 0 then
							DrawPicture(pictArray[1].pictH, pictRect)
						else if frameNo = 1 then
							DrawPicture(pictArray[6].pictH, pictRect)
						else
							DrawPicture(pictArray[7].pictH, pictRect);
				end;
			end;
	end;

{----------------------------}

	procedure DrawPellets;

		var
			positionLeft, positionTop, index: integer;
			pictRect: Rect;

	begin
		case gameMenus.windowSelection of
			Small: 
				begin
					with pelletInfo do
						begin

							for index := 0 to 25 do
								begin
									positionLeft := 134 + round(index * 9.5);
									positionTop := 51;
									Setrect(pelletRectArrayH[index], positionLeft, positionTop, positionLeft + pictSize, positionTop + pictSize);
									DrawPicture(pictImage.pictH, pelletRectArrayH[index]);
								end;

							for index := 0 to 25 do
								begin
									positionLeft := 182;
									positionTop := 13 + round(index * 9.5);
									Setrect(pelletRectArrayV[index], positionLeft, positionTop, positionLeft + pictSize, positionTop + pictSize);
									DrawPicture(pictImage.pictH, pelletRectArrayV[index]);
								end;

						end;
				end;

			Medium: 
				begin
					with pelletInfo do
						begin

							for index := 0 to 25 do
								begin
									positionLeft := 143 + (index * 14);
									positionTop := 76;
									Setrect(pelletRectArrayH[index], positionLeft, positionTop, positionLeft + pictSize, positionTop + pictSize);
									DrawPicture(pictImage.pictH, pelletRectArrayH[index]);
								end;

							for index := 0 to 25 do
								begin
									positionLeft := 213;
									positionTop := 20 + (index * 14);
									Setrect(pelletRectArrayV[index], positionLeft, positionTop, positionLeft + pictSize, positionTop + pictSize);
									DrawPicture(pictImage.pictH, pelletRectArrayV[index]);
								end;

						end;
				end;

			Large: 
				begin

					with pelletInfo do
						begin

							for index := 0 to 25 do
								begin
									positionLeft := 215 + round(index * 23.3);
									positionTop := 125;
									Setrect(pelletRectArrayH[index], positionLeft, positionTop, positionLeft + pictSize, positionTop + pictSize);
									DrawPicture(pictImage.pictH, pelletRectArrayH[index]);
								end;

							for index := 0 to 25 do
								begin
									positionLeft := 333;
									positionTop := 32 + round(index * 23.3);
									Setrect(pelletRectArrayV[index], positionLeft, positionTop, positionLeft + pictSize, positionTop + pictSize);
									DrawPicture(pictImage.pictH, pelletRectArrayV[index]);
								end;

						end;
				end;

		end;
	end;

{----------------------------}

	procedure DrawBackdrop;

	begin
		with backgroundPict do
			begin
				FillRect(pictRect, white);
				DrawPicture(pictH, pictRect);
			end;
	end;

{----------------------------}

	procedure DrawScreen (entireFlag: boolean);

		var
			offscreenPixMap, windowPixMap: PixMap;
			windowBitMap: BitMap;
			screenRect, gameRect: Rect;
			currPort: GWorldPtr;
			currDevice: GDHandle;

	begin
		GetGWorld(currPort, currDevice);
		SetGWorld(gameWindow.offGWorld, nil);
		offscreenPixMap := GetGWorldPixMap(gameWindow.offGWorld)^^;
		if (LockPixels(GetGWorldPixMap(gameWindow.offGWorld))) then
			begin

				if entireFlag then
					begin
{ draw all game elements onto offscreen PixMap }
{ background color }
						FillRect(gameWindow.offGWorld^.portRect, black);
						DrawBackdrop;
						DrawPellets;
						DrawPac(offscreenPixMap);
						DrawStats;
					end
				else
					begin
{ draw only applicable game elements onto offscreen PixMap }
						DrawPac(offscreenPixMap);
						DrawStats;
					end;

				UnlockPixels(GetGWorldPixMap(gameWindow.offGWorld));
			end;

{ select gameWindow }
		SetGWorld(currPort, currDevice);
{ background color }
		if entireFlag then
			FillRect(gameWindow.windowPtr^.portRect, black);

{ offscreenPixMap fully drawn, now copy it to the gameWindow PixMap }
		screenRect := gameWindow.windowPtr^.portRect;
		gameRect := backgroundPict.pictRect;

		if gameMenus.windowSelection = Small then
			begin
				windowBitMap := gameWindow.windowPtr^.portBits;
				if gameMenus.developerViewFlag then
					CopyBits(BitMapPtr(@offscreenPixMap)^, windowBitMap, screenRect, screenRect, srcCopy, nil)
				else
					CopyBits(BitMapPtr(@offscreenPixMap)^, windowBitMap, gameRect, gameRect, srcCopy, nil);
			end

		else if gameMenus.windowSelection = Medium then
			begin
				windowPixMap := gameWindow.cWindowPtr^.portPixMap^^;
				if gameMenus.developerViewFlag then
					CopyBits(BitMapPtr(@offscreenPixMap)^, BitMapPtr(@windowPixMap)^, screenRect, screenRect, srcCopy, nil)
				else
					CopyBits(BitMapPtr(@offscreenPixMap)^, BitMapPtr(@windowPixMap)^, gameRect, gameRect, srcCopy, nil);
			end

		else if gameMenus.windowSelection = Large then
			begin
				windowPixMap := gameWindow.cWindowPtr^.portPixMap^^;
				if gameMenus.developerViewFlag then
					CopyBits(BitMapPtr(@offscreenPixMap)^, BitMapPtr(@windowPixMap)^, screenRect, screenRect, srcCopy, nil)
				else
					CopyBits(BitMapPtr(@offscreenPixMap)^, BitMapPtr(@windowPixMap)^, gameRect, gameRect, srcCopy, nil);
			end;

		appGlobals.lastDrawTick := TickCount;
		appGlobals.drawCycles := appGlobals.drawCycles + 1;
	end;

{----------------------------}

end.